@@ -54,7 +54,7 @@ class CORE_EXPORT QgsBrowserWatcher : public QFutureWatcher<QVector <QgsDataItem
  * \ingroup core
  * \class QgsBrowserModel
  *
- * A model for showing available data sources and other items in a structured
+ * \brief A model for showing available data sources and other items in a structured
  * tree.
  *
  * QgsBrowserModel is the foundation for the QGIS browser panel, and includes
@@ -92,7 +92,7 @@ class CORE_EXPORT QgsBrowserModel : public QAbstractItemModel
       PathRole = Qt::UserRole, //!< Item path used to access path in the tree, see QgsDataItem::mPath
       CommentRole = Qt::UserRole + 1, //!< Item comment
       SortRole, //!< Custom sort role, see QgsDataItem::sortKey()
-      ProviderKeyRole, //!< Data item provider key that created the item, \since QGIS 3.12
+      ProviderKeyRole, //!< Data item provider key that created the item, see QgsDataItem::providerKey() \since QGIS 3.12
     };
     // implemented methods from QAbstractItemModel for read-only access
 
