@@ -17,16 +17,16 @@
 
 #include "qgis_core.h"
 #include "qgis.h"
-#include "qgssymbol.h"
 #include "qgsrenderer.h"
-#include "qgsexpression.h"
-#include "qgscolorramp.h"
-#include "qgsdatadefinedsizelegend.h"
 
 #include <QHash>
 
 class QgsVectorLayer;
 class QgsStyle;
+class QgsDataDefinedSizeLegend;
+class QgsSymbol;
+class QgsExpression;
+class QgsColorRamp;
 
 /**
  * \ingroup core
@@ -131,7 +131,7 @@ class CORE_EXPORT QgsRendererCategory
     /**
      * Converts the category to a matching SLD rule, within the specified DOM document and \a element.
      */
-    void toSld( QDomDocument &doc, QDomElement &element, QgsStringMap props ) const;
+    void toSld( QDomDocument &doc, QDomElement &element, QVariantMap props ) const;
 
   protected:
     QVariant mValue;
@@ -161,6 +161,7 @@ class CORE_EXPORT QgsCategorizedSymbolRenderer : public QgsFeatureRenderer
      * can be added later by calling addCategory().
      */
     QgsCategorizedSymbolRenderer( const QString &attrName = QString(), const QgsCategoryList &categories = QgsCategoryList() );
+    ~QgsCategorizedSymbolRenderer() override;
 
     QgsSymbol *symbolForFeature( const QgsFeature &feature, QgsRenderContext &context ) const override;
     QgsSymbol *originalSymbolForFeature( const QgsFeature &feature, QgsRenderContext &context ) const override;
@@ -170,10 +171,11 @@ class CORE_EXPORT QgsCategorizedSymbolRenderer : public QgsFeatureRenderer
     bool filterNeedsGeometry() const override;
     QString dump() const override;
     QgsCategorizedSymbolRenderer *clone() const override SIP_FACTORY;
-    void toSld( QDomDocument &doc, QDomElement &element, const QgsStringMap &props = QgsStringMap() ) const override;
+    void toSld( QDomDocument &doc, QDomElement &element, const QVariantMap &props = QVariantMap() ) const override;
     QgsFeatureRenderer::Capabilities capabilities() override { return SymbolLevels | Filter; }
     QString filter( const QgsFields &fields = QgsFields() ) override;
     QgsSymbolList symbols( QgsRenderContext &context ) const override;
+    bool accept( QgsStyleEntityVisitorInterface *visitor ) const override;
 
     /**
      * Update all the symbols but leave categories and colors. This method also sets the source
@@ -322,6 +324,15 @@ class CORE_EXPORT QgsCategorizedSymbolRenderer : public QgsFeatureRenderer
      */
     QgsSymbol *sourceSymbol();
 
+    /**
+     * Returns the renderer's source symbol, which is the base symbol used for the each categories' symbol before applying
+     * the categories' color.
+     * \see setSourceSymbol()
+     * \see sourceColorRamp()
+     * \note Not available in Python bindings.
+     */
+    const QgsSymbol *sourceSymbol() const SIP_SKIP;
+
     /**
      * Sets the source symbol for the renderer, which is the base symbol used for the each categories' symbol before applying
      * the categories' color.
@@ -338,6 +349,14 @@ class CORE_EXPORT QgsCategorizedSymbolRenderer : public QgsFeatureRenderer
      */
     QgsColorRamp *sourceColorRamp();
 
+    /**
+     * Returns the source color ramp, from which each categories' color is derived.
+     * \see setSourceColorRamp()
+     * \see sourceSymbol()
+     * \note Not available in Python bindings.
+     */
+    const QgsColorRamp *sourceColorRamp() const SIP_SKIP;
+
     /**
      * Sets the source color ramp.
       * \param ramp color ramp. Ownership is transferred to the renderer
@@ -361,7 +380,7 @@ class CORE_EXPORT QgsCategorizedSymbolRenderer : public QgsFeatureRenderer
 
     /**
      * creates a QgsCategorizedSymbolRenderer from an existing renderer.
-     * \returns a new renderer if the conversion was possible, otherwise 0.
+     * \returns a new renderer if the conversion was possible, otherwise NULLPTR.
      * \since QGIS 2.5
      */
     static QgsCategorizedSymbolRenderer *convertFromRenderer( const QgsFeatureRenderer *renderer ) SIP_FACTORY;
