@@ -21,7 +21,7 @@
 #include <QItemDelegate>
 #include <QFile>
 #include "qgis_gui.h"
-#include "qgis.h"
+#include "qgis_sip.h"
 
 class QMimeData;
 class QgsPanelWidget;
@@ -29,7 +29,7 @@ class QgsPanelWidget;
 /**
  * \ingroup gui
  * \class QgsColorSwatchDelegate
- * A delegate for showing a color swatch in a list
+ * \brief A delegate for showing a color swatch in a list
  * \see QgsColorSchemeList
  * \since QGIS 2.5
  */
@@ -61,7 +61,7 @@ class GUI_EXPORT QgsColorSwatchDelegate : public QAbstractItemDelegate
 /**
  * \ingroup gui
  * \class QgsColorSchemeModel
- * A model for colors in a color scheme
+ * \brief A model for colors in a color scheme
  * \see QgsColorSchemeList
  * \since QGIS 2.5
  */
@@ -128,13 +128,13 @@ class GUI_EXPORT QgsColorSchemeModel: public QAbstractItemModel
      * Add a color to the list
      * \param color color to add
      * \param label label for color
-     * \param allowDuplicate set to true to allow duplicate colors to be added (colors which are already present in the list)
+     * \param allowDuplicate set to TRUE to allow duplicate colors to be added (colors which are already present in the list)
      */
     void addColor( const QColor &color, const QString &label = QString(), bool allowDuplicate = false );
 
     /**
      * Returns whether the color scheme model has been modified
-     * \returns true if colors have been modified
+     * \returns TRUE if colors have been modified
      */
     bool isDirty() const { return mIsDirty; }
 
@@ -156,7 +156,7 @@ class GUI_EXPORT QgsColorSchemeModel: public QAbstractItemModel
 /**
  * \ingroup gui
  * \class QgsColorSchemeList
- * An editable list of color swatches, taken from an associated QgsColorScheme.
+ * \brief An editable list of color swatches, taken from an associated QgsColorScheme.
  * \see QgsColorSchemeList
  * \since QGIS 2.5
  */
@@ -198,7 +198,7 @@ class GUI_EXPORT QgsColorSchemeList: public QTreeView
 
     /**
      * Returns whether the color scheme list has been modified
-     * \returns true if colors have been modified
+     * \returns TRUE if colors have been modified
      */
     bool isDirty() const;
 
@@ -229,7 +229,7 @@ class GUI_EXPORT QgsColorSchemeList: public QTreeView
      * Adds a color to the list
      * \param color color to add
      * \param label optional label for color
-     * \param allowDuplicate set to true to allow duplicate colors to be added, ie colors which already exist in the list
+     * \param allowDuplicate set to TRUE to allow duplicate colors to be added, ie colors which already exist in the list
      */
     void addColor( const QColor &color, const QString &label = QString(), bool allowDuplicate = false );
 
