@@ -26,14 +26,17 @@
 #include "qgsfeatureid.h"
 #include "qgsgeometry.h"
 #include "qgscustomdrophandler.h"
+#include "qgstemporalrangeobject.h"
+#include "qgsmapcanvasinteractionblocker.h"
+#include "qgsproject.h"
 
 #include <QDomDocument>
 #include <QGraphicsView>
-#include <QtCore>
 
 #include "qgsmapsettings.h" // TEMPORARY
 #include "qgsprevieweffect.h" //for QgsPreviewEffect::PreviewMode
 
+#include <QTimer>
 #include <QGestureEvent>
 #include "qgis_gui.h"
 
@@ -56,6 +59,7 @@ class QgsHighlight;
 class QgsVectorLayer;
 
 class QgsLabelingResults;
+
 class QgsMapRendererCache;
 class QgsMapRendererQImageJob;
 class QgsMapSettings;
@@ -67,9 +71,15 @@ class QgsRubberBand;
 class QgsMapCanvasAnnotationItem;
 class QgsReferencedRectangle;
 
+class QgsTemporalController;
+
+class QMenu;
+class QgsMapMouseEvent;
+
+
 /**
  * \ingroup gui
- * Map canvas is a class for displaying all GIS data types on a canvas.
+ * \brief Map canvas is a class for displaying all GIS data types on a canvas.
  */
 
 class GUI_EXPORT QgsMapCanvas : public QGraphicsView
@@ -122,6 +132,22 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
      */
     const QgsMapSettings &mapSettings() const SIP_KEEPREFERENCE;
 
+    /**
+     * Sets the temporal controller, tQgsMapCanvasInteractionBlockerhis controller will be used to
+     * update the canvas temporal range.
+     *
+     * \since QGIS 3.14
+     */
+    void setTemporalController( QgsTemporalController *controller );
+
+    /**
+     * Gets access to the temporal controller that will be used to
+     * update the canvas temporal range.
+     *
+     * \since QGIS 3.14
+     */
+    const QgsTemporalController *temporalController() const;
+
     /**
      * sets destination coordinate reference system
      * \since QGIS 2.4
@@ -263,27 +289,35 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
     //! Zoom to the next extent (view)
     void zoomToNextExtent();
 
-    // ! Clears the list of extents and sets current extent as first item
+    //! Clears the list of extents and sets current extent as first item
     void clearExtentHistory();
 
 
     /**
      * Set canvas extent to the bounding box of a set of features
-        \param layer the vector layer
-        \param ids the feature ids*/
+     * \param layer the vector layer
+     * \param ids the feature ids
+    */
     void zoomToFeatureIds( QgsVectorLayer *layer, const QgsFeatureIds &ids );
 
     /**
      * Centers canvas extent to feature ids
      * \param layer the vector layer
      * \param ids the feature ids
-     * \param alwaysRecenter if false, the canvas is recentered only if the bounding box is not contained within the current extent
+     * \param alwaysRecenter if FALSE, the canvas is recentered only if the bounding box is not contained within the current extent
      */
     void panToFeatureIds( QgsVectorLayer *layer, const QgsFeatureIds &ids, bool alwaysRecenter = true );
 
     //! Pan to the selected features of current (vector) layer keeping same extent.
     void panToSelected( QgsVectorLayer *layer = nullptr );
 
+    /**
+     * Pan to the combined extent of the selected features of all provided (vector) layers.
+     * \param layers A list of layers
+     * \since QGIS 3.18
+     */
+    void panToSelected( const QList<QgsMapLayer *> &layers );
+
     /**
      * Causes a set of features with matching \a ids from a vector \a layer to flash
      * within the canvas.
@@ -331,6 +365,21 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
     //! Returns the currently active tool
     QgsMapTool *mapTool();
 
+    /**
+     * Sets the \a project linked to this canvas.
+     *
+     * \since QGIS 3.14
+     */
+    void setProject( QgsProject *project );
+
+    /**
+     * Returns the project linked to this canvas.
+     * The returned value may be NULLPTR.
+     *
+     * \since QGIS 3.14
+     */
+    QgsProject *project();
+
     //! Write property of QColor bgColor.
     void setCanvasColor( const QColor &_newVal );
     //! Read property of QColor bgColor.
@@ -526,7 +575,8 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
      * Enables a preview mode for the map canvas
      * \param previewEnabled set to TRUE to enable a preview mode
      * \see setPreviewMode
-     * \since QGIS 2.3 */
+     * \since QGIS 2.3
+    */
     void setPreviewModeEnabled( bool previewEnabled );
 
     /**
@@ -534,7 +584,8 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
      * \returns TRUE if a preview mode is currently enabled
      * \see setPreviewModeEnabled
      * \see previewMode
-     * \since QGIS 2.3 */
+     * \since QGIS 2.3
+    */
     bool previewModeEnabled() const;
 
     /**
@@ -544,7 +595,8 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
      * \see previewMode
      * \see setPreviewModeEnabled
      * \see previewModeEnabled
-     * \since QGIS 2.3 */
+     * \since QGIS 2.3
+    */
     void setPreviewMode( QgsPreviewEffect::PreviewMode mode );
 
     /**
@@ -553,7 +605,8 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
      * \returns preview mode for map canvas
      * \see setPreviewMode
      * \see previewModeEnabled
-     * \since QGIS 2.3 */
+     * \since QGIS 2.3
+    */
     QgsPreviewEffect::PreviewMode previewMode() const;
 
     /**
@@ -617,12 +670,14 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
 
     /**
      * Sets the segmentation tolerance applied when rendering curved geometries
-    \param tolerance the segmentation tolerance*/
+     * \param tolerance the segmentation tolerance
+    */
     void setSegmentationTolerance( double tolerance );
 
     /**
      * Sets segmentation tolerance type (maximum angle or maximum difference between curve and approximation)
-    \param type the segmentation tolerance typename*/
+     * \param type the segmentation tolerance typename
+    */
     void setSegmentationToleranceType( QgsAbstractGeometry::SegmentationToleranceType type );
 
     /**
@@ -684,6 +739,55 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
      */
     void setCustomDropHandlers( const QVector<QPointer<QgsCustomDropHandler >> &handlers ) SIP_SKIP;
 
+    /**
+     * Set datetime \a range for the map canvas.
+     *
+     * The temporalRangeChanged() signal will be emitted if the temporal range has been changed.
+     *
+     * \note Calling setTemporalRange() does not automatically trigger a map refresh.
+     *
+     * \see temporalRange()
+     * \since QGIS 3.14
+    */
+    void setTemporalRange( const QgsDateTimeRange &range );
+
+    /**
+     * Returns map canvas datetime range.
+     *
+     * \see setTemporalRange()
+     * \since QGIS 3.14
+    */
+    const QgsDateTimeRange &temporalRange() const;
+
+    /**
+     * Installs an interaction \a blocker onto the canvas, which may prevent certain map canvas
+     * interactions from occurring.
+     *
+     * The caller retains ownership of \a blocker, and must correctly call removeInteractionBlocker()
+     * before deleting the object.
+     *
+     * \see allowInteraction()
+     * \see removeInteractionBlocker()
+     * \since QGIS 3.14
+     */
+    void installInteractionBlocker( QgsMapCanvasInteractionBlocker *blocker );
+
+    /**
+     * Removes an interaction \a blocker from the canvas.
+     *
+     * \see allowInteraction()
+     * \see installInteractionBlocker()
+     * \since QGIS 3.14
+     */
+    void removeInteractionBlocker( QgsMapCanvasInteractionBlocker *blocker );
+
+    /**
+     * Returns TRUE if the specified \a interaction is currently permitted on the canvas.
+     *
+     * \since QGIS 3.14
+     */
+    bool allowInteraction( QgsMapCanvasInteractionBlocker::Interaction interaction ) const;
+
   public slots:
 
     //! Repaints the canvas map
@@ -744,9 +848,11 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
      * Sets the factor of magnification to apply to the map canvas. Indeed, we
      * increase/decrease the DPI of the map settings according to this factor
      * in order to render marker point, labels, ... bigger.
+     * \param factor The factor of magnification
+     * \param center Optional point to re-center the map
      * \since QGIS 2.16
      */
-    void setMagnificationFactor( double factor );
+    void setMagnificationFactor( double factor, const QgsPointXY *center = nullptr );
 
     /**
      * Lock the scale, so zooming can be performed using magnication
@@ -767,6 +873,13 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
      */
     void zoomToSelected( QgsVectorLayer *layer = nullptr );
 
+    /**
+     * Zoom to the combined extent of the selected features of all provided (vector) layers.
+     * \param layers A list of layers
+     * \since QGIS 3.18
+     */
+    void zoomToSelected( const QList<QgsMapLayer *> &layers );
+
     /**
      * Set a list of resolutions (map units per pixel) to which to "snap to" when zooming the map
      * \param resolutions A list of resolutions
@@ -774,6 +887,16 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
      */
     void setZoomResolutions( const QList<double> &resolutions ) { mZoomResolutions = resolutions; }
 
+    /**
+     * Returns the zoom in factor.
+     */
+    double zoomInFactor() const;
+
+    /**
+     * Returns the zoom in factor.
+     */
+    double zoomOutFactor() const;
+
     /**
      * \returns List of resolutions to which to "snap to" when zooming the map
      * \see setZoomResolutions()
@@ -781,6 +904,26 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
      */
     const QList<double> &zoomResolutions() const { return mZoomResolutions; }
 
+    /**
+     * Returns the range of z-values which will be visible in the map.
+     *
+     * \see setZRange()
+     * \see zRangeChanged()
+     *
+     * \since QGIS 3.18
+     */
+    QgsDoubleRange zRange() const;
+
+    /**
+     * Sets the \a range of z-values which will be visible in the map.
+     *
+     * \see zRange()
+     * \see zRangeChanged()
+     *
+     * \since QGIS 3.18
+     */
+    void setZRange( const QgsDoubleRange &range );
+
   private slots:
     //! called when current maptool is destroyed
     void mapToolDestroyed();
@@ -796,12 +939,15 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
     void refreshMap();
 
     void mapThemeChanged( const QString &theme );
+    //! Renames the active map theme called \a theme to \a newTheme
+    void mapThemeRenamed( const QString &theme, const QString &newTheme );
 
   signals:
 
     /**
      * Emits current mouse position
-        \note changed in 1.3 */
+     * \note changed in 1.3
+    */
     void xyCoordinates( const QgsPointXY &p );
 
     //! Emitted when the scale of the map changes
@@ -828,6 +974,8 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
      */
     void canvasColorChanged();
 
+    // TODO: deprecate when decorations are reimplemented as map canvas items
+
     /**
      * Emitted when the canvas has rendered.
      * Passes a pointer to the painter on which the map was drawn. This is
@@ -835,10 +983,6 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
      * rendered.  Passing the painter allows plugins to work when the map is
      * being rendered onto a pixmap other than the mapCanvas own pixmap member.
      *
-     */
-
-    /**
-     * TODO: deprecate when decorations are reimplemented as map canvas items
      * - anything related to rendering progress is not visible outside of map canvas
      * - additional drawing shall be done directly within the renderer job or independently as a map canvas item
      */
@@ -852,6 +996,12 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
     //! Emitted when the canvas is about to be rendered.
     void renderStarting();
 
+    /**
+     * Emitted when the pending map refresh has been canceled
+     * \since QGIS 3.18
+     */
+    void mapRefreshCanceled();
+
     //! Emitted when a new set of layers has been received
     void layersChanged();
 
@@ -938,6 +1088,31 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
      */
     void tapAndHoldGestureOccurred( const QgsPointXY &mapPoint, QTapAndHoldGesture *gesture );
 
+    /**
+     * Emitted when the map canvas temporal range changes.
+     *
+     * \since QGIS 3.14
+     */
+    void temporalRangeChanged();
+
+    /**
+     * Emitted when the map canvas z (elevation) range changes.
+     *
+     * \see zRange()
+     * \see setZRange()
+     *
+     * \since QGIS 3.18
+     */
+    void zRangeChanged();
+
+    /**
+     * Emitted before the map canvas context menu will be shown.
+     * Can be used to extend the context menu.
+     *
+     * \since QGIS 3.16
+     */
+    void contextMenuAboutToShow( QMenu *menu, QgsMapMouseEvent *event );
+
   protected:
 
     bool event( QEvent *e ) override;
@@ -991,18 +1166,49 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
 
   private:
 
+    // Restore scale RAII
+    class ScaleRestorer
+    {
+      public:
+        ScaleRestorer( QgsMapCanvas *canvas ):
+          mCanvas( canvas )
+        {
+          mLockedScale = mCanvas->mapSettings().scale();
+        };
+
+        ~ScaleRestorer()
+        {
+          QgsRectangle newExtent = mCanvas->mapSettings().extent();
+          newExtent.scale( mLockedScale / mCanvas->mapSettings().scale() );
+          mCanvas->mSettings.setExtent( newExtent );
+        };
+
+      private:
+        QgsMapCanvas *mCanvas;
+        double mLockedScale;
+    };
+
     //! encompases all map settings necessary for map rendering
     QgsMapSettings mSettings;
 
     //! owns pixmap with rendered map and controls rendering
     QgsMapCanvasMap *mMap = nullptr;
 
+    /**
+     * Temporal controller for tracking update of temporal objects
+     * which relates with canvas
+     */
+    QgsTemporalController *mController = nullptr;
+
     //! Flag indicating if the map canvas is frozen.
     bool mFrozen = false;
 
     //! Flag that allows squashing multiple refresh() calls into just one delayed rendering job
     bool mRefreshScheduled = false;
 
+    //! Flag that triggers a refresh after an ongoing rendering job triggered by autoRefresh
+    bool mRefreshAfterJob = false;
+
     //! determines whether user has requested to suppress rendering
     bool mRenderFlag = true;
 
@@ -1018,6 +1224,9 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
     //! previous tool if current is for zooming/panning
     QgsMapTool *mLastNonZoomMapTool = nullptr;
 
+    //! Pointer to project linked to this canvas
+    QgsProject *mProject = nullptr;
+
     //! recently used extent
     QList <QgsRectangle> mLastExtent;
     int mLastExtentIndex = -1;
@@ -1093,6 +1302,10 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
     QgsDistanceArea mDa;
     QList<double> mZoomResolutions;
 
+    QList< QgsMapCanvasInteractionBlocker * > mInteractionBlockers;
+
+    int mBlockItemPositionUpdates = 0;
+
     /**
      * Returns the last cursor position on the canvas in geographical coordinates
      * \since QGIS 3.4
@@ -1121,13 +1334,22 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
 
     /**
      * Returns bounding box of feature list (in canvas coordinates)
-        \param ids feature id list
-        \param layer the layer
-        \param bbox out: bounding box
-        \param errorMsg error message in case of error
-        \returns true in case of success*/
+     * \param ids feature id list
+     * \param layer the layer
+     * \param bbox out: bounding box
+     * \param errorMsg error message in case of error
+     * \returns TRUE in case of success
+    */
     bool boundingBoxOfFeatureIds( const QgsFeatureIds &ids, QgsVectorLayer *layer, QgsRectangle &bbox, QString &errorMsg ) const;
 
+    /**
+     * Rerturns the optimal extent for a point \a layer and a given \a center point in canvas CRS.
+     * This will return an extent combined of the center and the closest point in the layer.
+     * The extent can be scaled with a \a scale factor.
+     * The returned extent might be an empty rect if it cannot be determnined.
+     */
+    QgsRectangle optimalExtentForPointLayer( QgsVectorLayer *layer, const QgsPointXY &center, int scaleFactor = 5 );
+
     void setLayersPrivate( const QList<QgsMapLayer *> &layers );
 
     void startPreviewJobs();
@@ -1140,8 +1362,19 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
     bool panOperationInProgress();
 
     int nextZoomLevel( const QList<double> &resolutions, bool zoomIn = true ) const;
-    double zoomInFactor() const;
-    double zoomOutFactor() const;
+
+    /**
+     * Make sure to remove any rendered images of temporal-enabled layers from cache (does nothing if cache is not enabled)
+     * \since QGIS 3.14
+     */
+    void clearTemporalCache();
+
+    /**
+     * Removes any rendered images of elevation aware layers from cache
+     */
+    void clearElevationCache();
+
+    void showContextMenu( QgsMapMouseEvent *event );
 
     friend class TestQgsMapCanvas;
 
