@@ -16,63 +16,73 @@
 #ifndef QGSMAPRENDERERPARALLELJOB_H
 #define QGSMAPRENDERERPARALLELJOB_H
 
+#include "qgis_core.h"
+#include "qgis_sip.h"
 #include "qgsmaprendererjob.h"
 
-/** \ingroup core
- * Job implementation that renders all layers in parallel.
+/**
+ * \ingroup core
+ * \brief Job implementation that renders all layers in parallel.
  *
  * The resulting map image can be retrieved with renderedImage() function.
  * It is safe to call that function while rendering is active to see preview of the map.
  *
- * @note added in 2.4
+ * \since QGIS 2.4
  */
 class CORE_EXPORT QgsMapRendererParallelJob : public QgsMapRendererQImageJob
 {
     Q_OBJECT
   public:
-    QgsMapRendererParallelJob( const QgsMapSettings& settings );
-    ~QgsMapRendererParallelJob();
+    QgsMapRendererParallelJob( const QgsMapSettings &settings );
+    ~QgsMapRendererParallelJob() override;
 
-    virtual void start() override;
-    virtual void cancel() override;
-    virtual void waitForFinished() override;
-    virtual bool isActive() const override;
+    void start() override;
+    void cancel() override;
+    void cancelWithoutBlocking() override;
+    void waitForFinished() override;
+    bool isActive() const override;
 
-    virtual QgsLabelingResults* takeLabelingResults() override;
+    bool usedCachedLabels() const override;
+    QgsLabelingResults *takeLabelingResults() SIP_TRANSFER override;
 
     // from QgsMapRendererJobWithPreview
-    virtual QImage renderedImage() override;
+    QImage renderedImage() override;
 
-  protected slots:
+  private slots:
     //! layers are rendered, labeling is still pending
     void renderLayersFinished();
+    //! the second pass of layer rendering is finished
+    void renderLayersSecondPassFinished();
     //! all rendering is finished, including labeling
     void renderingFinished();
 
-  protected:
+  private:
 
-    static void renderLayerStatic( LayerRenderJob& job );
-    static void renderLabelsStatic( QgsMapRendererParallelJob* self );
-
-  protected:
+    //! \note not available in Python bindings
+    static void renderLayerStatic( LayerRenderJob &job ) SIP_SKIP;
+    //! \note not available in Python bindings
+    static void renderLabelsStatic( QgsMapRendererParallelJob *self ) SIP_SKIP;
 
     QImage mFinalImage;
 
-    //! @note not available in Python bindings
-    enum { Idle, RenderingLayers, RenderingLabels } mStatus;
+    //! \note not available in Python bindings
+    enum { Idle, RenderingLayers, RenderingSecondPass, RenderingLabels } mStatus SIP_SKIP;
 
     QFuture<void> mFuture;
     QFutureWatcher<void> mFutureWatcher;
 
     LayerRenderJobs mLayerJobs;
+    LabelRenderJob mLabelJob;
+
+    LayerRenderJobs mSecondPassLayerJobs;
+    QFuture<void> mSecondPassFuture;
+    QFutureWatcher<void> mSecondPassFutureWatcher;
 
-    //! Old labeling engine
-    QgsPalLabeling* mLabelingEngine;
     //! New labeling engine
-    QgsLabelingEngineV2* mLabelingEngineV2;
-    QgsRenderContext mLabelingRenderContext;
+    std::unique_ptr< QgsLabelingEngine > mLabelingEngineV2;
     QFuture<void> mLabelingFuture;
     QFutureWatcher<void> mLabelingFutureWatcher;
+
 };
 
 
