@@ -40,7 +40,7 @@ class QgsMessageBarItem;
 
 /**
  * \ingroup gui
- * A bar for displaying non-blocking messages to the user.
+ * \brief A bar for displaying non-blocking messages to the user.
  *
  * QgsMessageBar is a reusable widget which allows for providing feedback to users in
  * a non-intrusive way. Messages are shown in a horizontal bar widget, which is styled
@@ -131,9 +131,10 @@ class GUI_EXPORT QgsMessageBar: public QFrame
      * how the message bar is styled.
      *
      * The optional \a duration argument can be used to specify the message timeout in seconds. If \a duration
-     * is set to 0, then the message must be manually dismissed by the user.
+     * is set to 0, then the message must be manually dismissed by the user. Since QGIS 3.18, a duration of -1 indicates that
+     * the default timeout for the message \a level should be used.
      */
-    void pushMessage( const QString &text, Qgis::MessageLevel level = Qgis::Info, int duration = 5 );
+    void pushMessage( const QString &text, Qgis::MessageLevel level = Qgis::Info, int duration = -1 );
 
     /**
      * A convenience method for pushing a message with the specified \a title and \a text to the bar.
@@ -142,9 +143,10 @@ class GUI_EXPORT QgsMessageBar: public QFrame
      * how the message bar is styled.
      *
      * The optional \a duration argument can be used to specify the message timeout in seconds. If \a duration
-     * is set to 0, then the message must be manually dismissed by the user.
+     * is set to 0, then the message must be manually dismissed by the user. Since QGIS 3.18, a duration of -1 indicates that
+     * the default timeout for the message \a level should be used.
      */
-    void pushMessage( const QString &title, const QString &text, Qgis::MessageLevel level = Qgis::Info, int duration = 5 );
+    void pushMessage( const QString &title, const QString &text, Qgis::MessageLevel level = Qgis::Info, int duration = -1 );
 
     /**
      * A convenience method for pushing a message with the specified \a title and \a text to the bar. Additional
@@ -154,9 +156,10 @@ class GUI_EXPORT QgsMessageBar: public QFrame
      * how the message bar is styled.
      *
      * The optional \a duration argument can be used to specify the message timeout in seconds. If \a duration
-     * is set to 0, then the message must be manually dismissed by the user.
+     * is set to 0, then the message must be manually dismissed by the user. Since QGIS 3.18, a duration of -1 indicates that
+     * the default timeout for the message \a level should be used.
      */
-    void pushMessage( const QString &title, const QString &text, const QString &showMore, Qgis::MessageLevel level = Qgis::Info, int duration = 5 );
+    void pushMessage( const QString &title, const QString &text, const QString &showMore, Qgis::MessageLevel level = Qgis::Info, int duration = -1 );
 
     /**
      * Returns the current visible item, or NULLPTR if no item is shown.
@@ -170,6 +173,12 @@ class GUI_EXPORT QgsMessageBar: public QFrame
      */
     QList<QgsMessageBarItem *> items();
 
+    /**
+     * Returns the default timeout in seconds for timed messages of the specified \a level.
+     * \since QGIS 3.18
+     */
+    static int defaultMessageTimeout( Qgis::MessageLevel level = Qgis::None );
+
   signals:
 
     /**
