@@ -33,7 +33,6 @@
 #include <functional>
 #include "qgsvectorlayerexporter.h"
 #include "qgsabstractproviderconnection.h"
-#include "qgsabstractdatabaseproviderconnection.h"
 #include "qgsfields.h"
 #include "qgsexception.h"
 
@@ -42,10 +41,14 @@ class QgsDataItemProvider;
 class QgsTransaction;
 
 class QgsRasterDataProvider;
+class QgsMeshDataProvider;
+class QgsAbstractDatabaseProviderConnection;
+
+struct QgsMesh;
 
 /**
  * \ingroup core
- * Holds metadata about mesh driver
+ * \brief Holds metadata about mesh driver
  *
  * \since QGIS 3.12
  */
@@ -62,6 +65,8 @@ class CORE_EXPORT QgsMeshDriverMetadata
     {
       CanWriteFaceDatasets = 1 << 0, //!< If the driver can persist datasets defined on faces
       CanWriteVertexDatasets = 1 << 1, //!< If the driver can persist datasets defined on vertices
+      CanWriteEdgeDatasets = 1 << 2, //!< If the driver can persist datasets defined on edges \since QGIS 3.14
+      CanWriteMeshData = 1 << 3, //!< If the driver can write mesh data on file \since QGIS 3.16
     };
 
     Q_ENUM( MeshDriverCapability )
@@ -77,10 +82,12 @@ class CORE_EXPORT QgsMeshDriverMetadata
      * \param name name/key of the driver
      * \param description short description of the driver
      * \param capabilities driver's capabilities
+     * \param writeDatasetOnFileSuffix suffix used to write datasets on file
      */
     QgsMeshDriverMetadata( const QString &name,
                            const QString &description,
-                           const MeshDriverCapabilities &capabilities );
+                           const MeshDriverCapabilities &capabilities,
+                           const QString &writeDatasetOnFileSuffix );
 
     /**
      * Returns the capabilities for this driver.
@@ -97,17 +104,23 @@ class CORE_EXPORT QgsMeshDriverMetadata
      */
     QString description() const;
 
+    /**
+     * Returns the suffix used to write datasets on file
+     */
+    QString writeDatasetOnFileSuffix() const;
+
   private:
     QString mName;
     QString mDescription;
     MeshDriverCapabilities mCapabilities;
+    QString mWriteDatasetOnFileSuffix;
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS( QgsMeshDriverMetadata::MeshDriverCapabilities )
 
 /**
  * \ingroup core
- * Holds data provider key, description, and associated shared library file or function pointer information.
+ * \brief Holds data provider key, description, and associated shared library file or function pointer information.
  *
  * Provider metadata refers either to providers which are loaded via libraries or
  * which are native providers that are included in the core QGIS installation
@@ -122,15 +135,40 @@ Q_DECLARE_OPERATORS_FOR_FLAGS( QgsMeshDriverMetadata::MeshDriverCapabilities )
  * library object.
  *
  */
-class CORE_EXPORT QgsProviderMetadata
+class CORE_EXPORT QgsProviderMetadata : public QObject
 {
+    Q_OBJECT
+
   public:
 
+    /**
+     * Indicates capabilities of the provider metadata implementation.
+     *
+     * \since QGIS 3.18
+     */
+    enum ProviderMetadataCapability
+    {
+      PriorityForUri = 1 << 0, //!< Indicates that the metadata can calculate a priority for a URI
+      LayerTypesForUri = 1 << 1, //!< Indicates that the metadata can determine valid layer types for a URI
+    };
+    Q_DECLARE_FLAGS( ProviderMetadataCapabilities, ProviderMetadataCapability )
+
+    /**
+     * Provider capabilities
+     *
+     * \since QGIS 3.18.1
+     */
+    enum ProviderCapability
+    {
+      FileBasedUris = 1 << 0, //!< Indicates that the provider can utilize URIs which are based on paths to files (as opposed to database or internet paths)
+    };
+    Q_DECLARE_FLAGS( ProviderCapabilities, ProviderCapability )
+
     /**
      * Typedef for data provider creation function.
      * \since QGIS 3.0
      */
-    SIP_SKIP typedef std::function < QgsDataProvider*( const QString &, const QgsDataProvider::ProviderOptions & ) > CreateDataProviderFunction;
+    SIP_SKIP typedef std::function < QgsDataProvider*( const QString &, const QgsDataProvider::ProviderOptions &, QgsDataProvider::ReadFlags & ) > CreateDataProviderFunction;
 
     /**
      * Constructor for provider metadata
@@ -165,6 +203,20 @@ class CORE_EXPORT QgsProviderMetadata
      */
     QString description() const;
 
+    /**
+     * Returns the provider metadata capabilities.
+     *
+     * \since QGIS 3.18
+     */
+    virtual QgsProviderMetadata::ProviderMetadataCapabilities capabilities() const;
+
+    /**
+     * Returns the provider's capabilities.
+     *
+     * \since QGIS 3.18.1
+     */
+    virtual QgsProviderMetadata::ProviderCapabilities providerCapabilities() const;
+
     /**
      * This returns the library file name
      *
@@ -201,10 +253,11 @@ class CORE_EXPORT QgsProviderMetadata
      */
     enum class FilterType
     {
-      FilterVector = 1,
-      FilterRaster,
-      FilterMesh,
-      FilterMeshDataset
+      FilterVector = 1, //!< Vector layers
+      FilterRaster, //!< Raster layers
+      FilterMesh, //!< Mesh layers
+      FilterMeshDataset, //!< Mesh datasets
+      FilterPointCloud, //!< Point clouds (since QGIS 3.18)
     };
 
     /**
@@ -223,11 +276,82 @@ class CORE_EXPORT QgsProviderMetadata
      */
     virtual QList<QgsMeshDriverMetadata> meshDriversMetadata();
 
+    /**
+     * Returns an integer representing the priority which this provider should have when opening
+     * a dataset with the specified \a uri.
+     *
+     * A larger priority means that the provider should be selected over others with a lower
+     * priority for the same URI.
+     *
+     * The default implementation returns 0 for all URIs.
+     *
+     * \warning Not all providers implement this functionality. Check whether capabilities() returns the
+     * ProviderMetadataCapability::PriorityForUri to determine whether a specific provider metadata object
+     * supports this method.
+     *
+     * \since QGIS 3.18
+     */
+    virtual int priorityForUri( const QString &uri ) const;
+
+    /**
+     * Returns a list of valid layer types which the provider can be used with when
+     * opening the specified \a uri.
+     *
+     * \warning Not all providers implement this functionality. Check whether capabilities() returns the
+     * ProviderMetadataCapability::LayerTypesForUri to determine whether a specific provider metadata object
+     * supports this method.
+     *
+     * \since QGIS 3.18
+     */
+    virtual QList< QgsMapLayerType > validLayerTypesForUri( const QString &uri ) const;
+
+    /**
+     * Returns TRUE if the specified \a uri is known by this provider to be something which should
+     * be blocklisted from the QGIS interface, e.g. an internal detail only.
+     *
+     * Specifically, this method can be utilized by the browser panel to hide noisy internal details
+     * by returning TRUE for URIs which are known to be sidecar files only, such as ".aux.xml" files
+     * or ".shp.xml" files, or the "ept-build.json" files which sit alongside Entwine "ept.json" point
+     * cloud sources.
+     *
+     * The default method returns FALSE for all URIs.
+     *
+     * \warning Returning TRUE from an implementation of this method indicates that ALL providers should
+     * ignore the specified \a uri, not just the provider associated with this metadata!
+     *
+     * \since QGIS 3.18
+     */
+    virtual bool uriIsBlocklisted( const QString &uri ) const;
+
     /**
      * Class factory to return a pointer to a newly created QgsDataProvider object
+     *
+     * \param uri the datasource uri
+     * \param options creation options
+     * \param flags creation flags, sing QGIS 3.16
+     *
      * \since QGIS 3.10
      */
-    virtual QgsDataProvider *createProvider( const QString &uri, const QgsDataProvider::ProviderOptions &options ) SIP_FACTORY;
+    virtual QgsDataProvider *createProvider( const QString &uri,
+        const QgsDataProvider::ProviderOptions &options,
+        QgsDataProvider::ReadFlags flags = QgsDataProvider::ReadFlags() ) SIP_FACTORY;
+
+    /**
+     * Sets the \a value into the \a uri \a parameter as a bool.
+     * eg. "yes" value will be saved as TRUE, 0 will be saved as FALSE
+     *
+     * \since QGIS 3.14
+     */
+    static void setBoolParameter( QVariantMap &uri, const QString &parameter, const QVariant &value );
+
+    /**
+     * Returns the \a parameter value in the \a uri as a bool.
+     * eg. "yes" value will be returned as TRUE, 0 will be returned as FALSE
+     *
+     * \since QGIS 3.14
+     */
+    static bool boolParameter( const QVariantMap &uri, const QString &parameter, bool defaultValue = false );
+
 
 #ifndef SIP_RUN
 
@@ -261,6 +385,16 @@ class CORE_EXPORT QgsProviderMetadata
       const QgsCoordinateReferenceSystem &crs,
       const QStringList &createOptions = QStringList() ) SIP_FACTORY;
 
+    /**
+     * Creates mesh data source, that is the mesh frame stored in file, memory or with other way (depending of the provider)
+     * \since QGIS 3.16
+     */
+    virtual bool createMeshData(
+      const QgsMesh &mesh,
+      const QString uri,
+      const QString &driverName,
+      const QgsCoordinateReferenceSystem &crs ) const;
+
     /**
      * Returns pyramid resampling methods available for provider
      * \since QGIS 3.10
@@ -270,11 +404,21 @@ class CORE_EXPORT QgsProviderMetadata
     /**
      * Breaks a provider data source URI into its component paths (e.g. file path, layer name).
      * \param uri uri string
-     * \returns map containing components. Standard components include "path", "layerName", "url".
+     * \returns map containing components. Standard components may include:
+     *
+     * - "path": file path
+     * - "layerName"
+     * - "url": base URL, for online services
+     * - "referer": referrer string, for HTTP requests
+     * - "host": hostname, for database services
+     * - "bounds": hardcoded layer bounds (as a QgsRectangle)
+     * - "crs": CRS definition
+     * - "authcfg": authentication configuration ID
+     *
      * \note this function may not be supported by all providers, an empty map will be returned in such case
      * \since QGIS 3.10
      */
-    virtual QVariantMap decodeUri( const QString &uri );
+    virtual QVariantMap decodeUri( const QString &uri ) const;
 
     /**
      * Reassembles a provider data source URI from its component paths (e.g. file path, layer name).
@@ -284,7 +428,7 @@ class CORE_EXPORT QgsProviderMetadata
      * \see decodeUri()
      * \since QGIS 3.12
      */
-    virtual QString encodeUri( const QVariantMap &parts );
+    virtual QString encodeUri( const QVariantMap &parts ) const;
 
     /**
      * Returns data item providers. Caller is responsible for ownership of the item providers
@@ -392,17 +536,19 @@ class CORE_EXPORT QgsProviderMetadata
      * the newly created connection is not automatically stored in the settings, call
      * saveConnection() to save it.
      * Ownership is transferred to the caller.
+     * \throws QgsProviderConnectionException
      * \see saveConnection()
      * \since QGIS 3.10
      */
-    virtual QgsAbstractProviderConnection *createConnection( const QString &uri, const QVariantMap &configuration ) SIP_FACTORY;
+    virtual QgsAbstractProviderConnection *createConnection( const QString &uri, const QVariantMap &configuration ) SIP_THROW( QgsProviderConnectionException ) SIP_FACTORY;
 
     /**
      * Creates a new connection by loading the connection with the given \a name from the settings.
      * Ownership is transferred to the caller.
+     * \throws QgsProviderConnectionException
      * \see findConnection()
      */
-    virtual QgsAbstractProviderConnection *createConnection( const QString &name );
+    virtual QgsAbstractProviderConnection *createConnection( const QString &name ) SIP_THROW( QgsProviderConnectionException ) SIP_FACTORY;
 
     /**
      * Removes the connection with the given \a name from the settings.
@@ -416,9 +562,45 @@ class CORE_EXPORT QgsProviderMetadata
      * Stores the connection in the settings
      * \param connection the connection to be stored in the settings
      * \param name the name under which the connection will be stored
+     * \throws QgsProviderConnectionException
      * \since QGIS 3.10
      */
-    virtual void saveConnection( const QgsAbstractProviderConnection *connection, const QString &name );
+    virtual void saveConnection( const QgsAbstractProviderConnection *connection, const QString &name ) SIP_THROW( QgsProviderConnectionException );
+
+#ifdef SIP_RUN
+    SIP_PYOBJECT __repr__();
+    % MethodCode
+    QString str = QStringLiteral( "<QgsProviderMetadata: %1>" ).arg( sipCpp->key() );
+    sipRes = PyUnicode_FromString( str.toUtf8().constData() );
+    % End
+#endif
+
+  signals:
+
+    /**
+     * Emitted when a connection with the specified \a name is created.
+     *
+     * \note Only providers which implement the connection handling API will emit this signal.
+     * \since QGIS 3.14
+     */
+    void connectionCreated( const QString &name );
+
+    /**
+     * Emitted when the connection with the specified \a name was deleted.
+     *
+     * \note Only providers which implement the connection handling API will emit this signal.
+     * \since QGIS 3.14
+     */
+    void connectionDeleted( const QString &name );
+
+    /**
+     * Emitted when the connection with the specified \a name is changed, e.g. the settings
+     * relating to the connection have been updated.
+     *
+     * \note Only providers which implement the connection handling API will emit this signal.
+     * \since QGIS 3.14
+     */
+    void connectionChanged( const QString &name );
 
   protected:
 
@@ -450,12 +632,14 @@ class CORE_EXPORT QgsProviderMetadata
       T_provider_conn conn( name );
       conn.remove( name );
       mProviderConnections.clear();
+      emit connectionDeleted( name );
     }
     virtual void saveConnectionProtected( const QgsAbstractProviderConnection *connection, const QString &name );
     //! Provider connections cache
     QMap<QString, QgsAbstractProviderConnection *> mProviderConnections;
 
 /// @endcond
+
 #endif
 
   private:
@@ -474,4 +658,7 @@ class CORE_EXPORT QgsProviderMetadata
 
 };
 
+Q_DECLARE_OPERATORS_FOR_FLAGS( QgsProviderMetadata::ProviderMetadataCapabilities )
+
+
 #endif //QGSPROVIDERMETADATA_H
