@@ -30,6 +30,9 @@
 #include "qgis_core.h"
 #include "qgis_sip.h"
 
+#include <vector>
+#include <memory>
+
 class QgsProviderMetadata;
 class QgsVectorLayer;
 class QgsCoordinateReferenceSystem;
@@ -39,7 +42,7 @@ class QgsRasterDataProvider;
 
 /**
  * \ingroup core
-  * A registry / canonical manager of data providers.
+  * \brief A registry / canonical manager of data providers.
   *
   * This is a Singleton class that manages data provider access.
   *
@@ -56,10 +59,11 @@ class CORE_EXPORT QgsProviderRegistry
 
   public:
 
+    // TODO QGIS 4 - either move to QgsAbstractDataSourceWidget or remove altogether
+
     /**
      * Different ways a source select dialog can be used
      */
-    // TODO QGIS 4 - either move to QgsAbstractDataSourceWidget or remove altogether
     enum WidgetMode
     {
 
@@ -87,7 +91,7 @@ class CORE_EXPORT QgsProviderRegistry
     //! Means of accessing canonical single instance
     static QgsProviderRegistry *instance( const QString &pluginPath = QString() );
 
-    virtual ~QgsProviderRegistry();
+    ~QgsProviderRegistry();
 
     /**
      * Returns path for the library of the provider.
@@ -110,24 +114,43 @@ class CORE_EXPORT QgsProviderRegistry
     //! Sets library directory where to search for plugins
     void setLibraryDirectory( const QDir &path );
 
+    /*
+     * IMPORTANT: While it seems like /Factory/ would be the correct annotation here, that's not
+     * the case.
+     * Paraphrasing Phil Thomson's advice on https://www.riverbankcomputing.com/pipermail/pyqt/2017-July/039450.html:
+     *
+     * "
+     * /Factory/ is used when the instance returned is guaranteed to be new to Python.
+     * In this case it isn't because it has already been seen when being returned by by the python function
+     * creating the provider subclass.
+     *
+     * (However for a different sub-class implemented in C++ then it would be the first time it was seen
+     * by Python so the /Factory/ on create() would be correct.)
+     *
+     * You might try using /TransferBack/ on createProvider() instead - that might be the best compromise.
+     * "
+     */
+
     /**
      * Creates a new instance of a provider.
      * \param providerKey identifier of the provider
      * \param dataSource  string containing data source for the provider
      * \param options provider options
+     * \param flags provider flags since QGIS 3.16
      * \returns new instance of provider or NULLPTR on error
      *
      * \see createRasterDataProvider()
      */
     QgsDataProvider *createProvider( const QString &providerKey,
                                      const QString &dataSource,
-                                     const QgsDataProvider::ProviderOptions &options = QgsDataProvider::ProviderOptions() ) SIP_FACTORY;
+                                     const QgsDataProvider::ProviderOptions &options = QgsDataProvider::ProviderOptions(),
+                                     QgsDataProvider::ReadFlags flags = QgsDataProvider::ReadFlags() ) SIP_TRANSFERBACK;
 
     /**
      * Returns the provider capabilities
-        \param providerKey identifier of the provider
-        \since QGIS 2.6
-        \deprecated QGIS 3.10 (use instead capabilities() method of individual data item provider)
+     * \param providerKey identifier of the provider
+     * \since QGIS 2.6
+     * \deprecated QGIS 3.10 (use instead capabilities() method of individual data item provider)
      */
     Q_DECL_DEPRECATED int providerCapabilities( const QString &providerKey ) const SIP_DEPRECATED;
 
@@ -144,7 +167,7 @@ class CORE_EXPORT QgsProviderRegistry
      * \see createProvider()
      * \since QGIS 3.10
      */
-    virtual QgsRasterDataProvider *createRasterDataProvider(
+    QgsRasterDataProvider *createRasterDataProvider(
       const QString &providerKey,
       const QString &uri,
       const QString &format,
@@ -280,69 +303,301 @@ class CORE_EXPORT QgsProviderRegistry
     QgsProviderMetadata *providerMetadata( const QString &providerKey ) const;
 
     /**
-     * Returns vector file filter string
-
-      Returns a string suitable for a QFileDialog of vector file formats
-      supported by all data providers.
-
-      This walks through all data providers appending calls to their
-      fileVectorFilters to a string, which is then returned.
-
-      \note
-
-      It'd be nice to eventually be raster/vector neutral.
+     * \ingroup core
+     *
+     * \brief Contains information pertaining to a candidate provider.
+     *
+     * \since QGIS 3.18
      */
-    virtual QString fileVectorFilters() const;
+    class CORE_EXPORT ProviderCandidateDetails
+    {
+
+      public:
+
+        /**
+         * Constructor for ProviderCandidateDetails, with the specified provider \a metadata and valid candidate \a layerTypes.
+         */
+        ProviderCandidateDetails( QgsProviderMetadata *metadata, const QList< QgsMapLayerType > &layerTypes )
+          : mMetadata( metadata )
+          , mLayerTypes( layerTypes )
+        {}
+
+        /**
+         * Returns the candidate provider metadata.
+         */
+        QgsProviderMetadata *metadata() const { return mMetadata; }
+
+        /**
+         * Returns a list of map layer types which are valid options for opening the
+         * target using this candidate provider.
+         */
+        QList<QgsMapLayerType> layerTypes() const { return mLayerTypes; }
+
+#ifdef SIP_RUN
+        SIP_PYOBJECT __repr__();
+        % MethodCode
+        QString str = QStringLiteral( "<QgsProviderRegistry.ProviderCandidateDetails: %1>" ).arg( sipCpp->metadata()->key() );
+        sipRes = PyUnicode_FromString( str.toUtf8().constData() );
+        % End
+#endif
+
+      private:
+        QgsProviderMetadata *mMetadata = nullptr;
+
+        QList< QgsMapLayerType > mLayerTypes;
+
+    };
 
     /**
-     * Returns raster file filter string
-
-      Returns a string suitable for a QFileDialog of raster file formats
-      supported by all data providers.
-
-      This walks through all data providers appending calls to their
-      buildSupportedRasterFileFilter to a string, which is then returned.
-
-      \note This replaces QgsRasterLayer::buildSupportedRasterFileFilter()
+     * Returns the details for the preferred provider(s) for opening the specified \a uri.
+     *
+     * The preferred provider is determined by comparing the priority returned by
+     * QgsProviderMetadata::priorityForUri() for all registered providers, and selecting
+     * the provider with the largest non-zero priority.
+     *
+     * An empty list may be returned, which indicates that no providers are available which
+     * returned a non-zero priority for the specified URI.
+     *
+     * In the case that multiple providers returned the same priority for the URI then
+     * all of these providers will be returned.
+     *
+     * \see shouldDeferUriForOtherProviders()
+     * \since QGIS 3.18
      */
-    virtual QString fileRasterFilters() const;
+    QList< QgsProviderRegistry::ProviderCandidateDetails > preferredProvidersForUri( const QString &uri ) const;
 
     /**
-     * Returns mesh file filter string
-
-      Returns a string suitable for a QFileDialog of mesh file formats
-      supported by all data providers.
-
-      This walks through all data providers appending calls to their
-      fileMeshFilters to a string, which is then returned.
-
-      \see fileMeshDatasetFilters()
-
-      \since QGIS 3.6
+     * \ingroup core
+     *
+     * \brief Contains information about unusable URIs which aren't handled by any registered providers.
+     *
+     * For example, if a QGIS install is built without the PDAL library then las/laz files are unusable.
+     * This class can then be used to construct friendly warnings to users advising them why the las/laz
+     * files cannot be used on their QGIS build.
+     *
+     * \since QGIS 3.18.1
      */
-    virtual QString fileMeshFilters() const;
+    class CORE_EXPORT UnusableUriDetails
+    {
+      public:
+
+        /**
+         * Constructor for UnusableUriDetails for the given \a uri, with the specified user-friendly, translated \a warning.
+         *
+         * The optional \a layerTypes argument can be used to specify layer types which are usually valid
+         * options for opening the URI.
+         */
+        UnusableUriDetails( const QString &uri = QString(), const QString &warning = QString(), const QList< QgsMapLayerType > &layerTypes = QList< QgsMapLayerType >() )
+          : uri( uri )
+          , warning( warning )
+          , layerTypes( layerTypes )
+        {}
+
+        /**
+         * URI which could not be handled.
+         */
+        QString uri;
+
+        /**
+         * Contains a short, user-friendly, translated message advising why the URI is not usable.
+         */
+        QString warning;
+
+        /**
+         * Contains a longer, user-friendly, translated message advising why the URI is not usable.
+         */
+        QString detailedWarning;
+
+        /**
+         * Contains a list of map layer types which are usually valid options for opening the
+         * target URI.
+         */
+        QList<QgsMapLayerType> layerTypes;
+
+#ifdef SIP_RUN
+        SIP_PYOBJECT __repr__();
+        % MethodCode
+        QString str = QStringLiteral( "<QgsProviderRegistry.UnusableUriDetails: %1>" ).arg( sipCpp->warning );
+        sipRes = PyUnicode_FromString( str.toUtf8().constData() );
+        % End
+#endif
+
+    };
 
     /**
-     * Returns mesh's dataset file filter string
-
-      Returns a string suitable for a QFileDialog of mesh datasets file formats
-      supported by all data providers.
-
-      This walks through all data providers appending calls to their
-      fileMeshFilters to a string, which is then returned.
-
-      \see fileMeshFilters()
-
-      \since QGIS 3.6
+     * \ingroup core
+     *
+     * \brief An interface used to handle unusable URIs which aren't handled by any registered providers, and construct
+     * user-friendly warnings as to why the URI is unusable.
+     *
+     * For example, if a QGIS install is built without the PDAL library then las/laz files are unusable.
+     * This class can then be used to construct friendly warnings to users advising them why the las/laz
+     * files cannot be used on their QGIS build.
+     *
+     * \since QGIS 3.18.1
      */
-    virtual QString fileMeshDatasetFilters() const;
+    class CORE_EXPORT UnusableUriHandlerInterface
+    {
+
+      public:
+
+        virtual ~UnusableUriHandlerInterface() = default;
+
+        /**
+         * Returns TRUE if the handle is an unusable URI handler for the specified \a uri.
+         */
+        virtual bool matchesUri( const QString &uri ) const = 0;
+
+        /**
+         * Returns the details for advising the user why the \a uri is not usable.
+         */
+        virtual UnusableUriDetails details( const QString &uri ) const = 0;
+
+    };
+
+    /**
+     * \brief Registers an unusable URI \a handler, used to handle unusable URIs which aren't
+     * handled by any registered providers, and construct user-friendly warnings as to why the URI is unusable.
+     *
+     * \return TRUE on success
+     *
+     * \note ownership of the UnusableUriHandlerInterface instance is transferred to the registry
+     *
+     * \since QGIS 3.18.1
+     */
+    bool registerUnusableUriHandler( UnusableUriHandlerInterface *handler SIP_TRANSFER );
+
+    /**
+     * Returns TRUE if the specified \a uri can potentially be handled by QGIS, if additional
+     * dependencies or build-time requirements are present.
+     *
+     * This can be used to show user-friendly warning messages advising them why a particular
+     * \a uri cannot be opened on their QGIS install. For example, if a QGIS install is built
+     * without the PDAL library then las/laz files are unusable, and this method can be used
+     * to retrieve a user-friendly warning as to why the las/laz files cannot be used on their
+     * QGIS build.
+     *
+     * \warning This method does not perform the test to actually determine if the given \a uri
+     * can be handled by any registered provider. It is assumed that prior to calling this method
+     * the caller has already determined in advance that the \a uri could not be handled.
+     *
+     * \param uri URI to test
+     * \param details will be populated with details allowing construction of a user-friendly
+     * warning message
+     *
+     * \returns TRUE if the \a uri was matched to a registered QgsProviderRegistry::UnusableUriHandlerInterface.
+     *
+     * \see registerUnusableUriHandler()
+     * \since QGIS 3.18.1
+     */
+    bool handleUnusableUri( const QString &uri, UnusableUriDetails &details SIP_OUT ) const;
+
+    /**
+     * Returns TRUE if the provider with matching \a providerKey should defer handling of
+     * the specified \a uri to another provider.
+     *
+     * This method tests whether any providers are listed as the preferred provider for \a uri
+     * (see preferredProvidersForUri()), and if so tests whether the specified provider is
+     * included in that preferred providers list. Returns TRUE only if the specified provider
+     * is calculated as one of the preferred providers for the URI.
+     *
+     * In the case that there is no registered preferred provider for the URI then FALSE will be
+     * returned, and the provider must use another metric to determine whether it should
+     * handle the URI.
+     *
+     * \see preferredProvidersForUri()
+     * \since QGIS 3.18
+     */
+    bool shouldDeferUriForOtherProviders( const QString &uri, const QString &providerKey ) const;
+
+    /**
+     * Returns TRUE if the specified \a uri is known by any registered provider to be something which should
+     * be blocklisted from the QGIS interface, e.g. an internal detail only.
+     *
+     * Specifically, this method can be utilized by the browser panel to hide noisy internal details
+     * for URIs which are known to be sidecar files only, such as ".aux.xml" files or ".shp.xml" files,
+     * or the "ept-build.json" files which sit alongside Entwine "ept.json" point cloud sources.
+     *
+     * This method tests whether any of the registered providers return TRUE for the their
+     * QgsProviderMetadata::uriIsBlocklisted() implementation for the specified URI.
+     *
+     * \since QGIS 3.18
+     */
+    bool uriIsBlocklisted( const QString &uri ) const;
+
+    /**
+     * Returns a file filter string for supported vector files.
+     *
+     * Returns a string suitable for a QFileDialog of vector file formats
+     * supported by all data providers.
+     *
+     * \see fileRasterFilters()
+     * \see fileMeshFilters()
+     * \see filePointCloudFilters()
+     */
+    QString fileVectorFilters() const;
+
+    /**
+     * Returns a file filter string for supported raster files.
+     *
+     * Returns a string suitable for a QFileDialog of raster file formats
+     * supported by all data providers.
+     *
+     * \note This replaces QgsRasterLayer::buildSupportedRasterFileFilter()
+     *
+     * \see fileVectorFilters()
+     * \see fileMeshFilters()
+     * \see filePointCloudFilters()
+     */
+    QString fileRasterFilters() const;
+
+    /**
+     * Returns a file filter string for supported mesh files.
+     *
+     * Returns a string suitable for a QFileDialog of mesh file formats
+     * supported by all data providers.
+     *
+     * \see fileMeshDatasetFilters()
+     * \see fileRasterFilters()
+     * \see fileVectorFilters()
+     * \see filePointCloudFilters()
+     *
+     * \since QGIS 3.6
+     */
+    QString fileMeshFilters() const;
+
+    /**
+     * Returns a file filter string for supported mesh dataset files.
+     *
+     * Returns a string suitable for a QFileDialog of mesh datasets file formats
+     * supported by all data providers.
+     *
+     * \see fileMeshFilters()
+     *
+     * \since QGIS 3.6
+     */
+    QString fileMeshDatasetFilters() const;
+
+    /**
+     * Returns a file filter string for supported point clouds.
+     *
+     * Returns a string suitable for a QFileDialog of point cloud file formats
+     * supported by all data providers.
+     *
+     * \see fileMeshFilters()
+     * \see fileRasterFilters()
+     * \see fileVectorFilters()
+     *
+     * \since QGIS 3.18
+     */
+    QString filePointCloudFilters() const;
 
     //! Returns a string containing the available database drivers
-    virtual QString databaseDrivers() const;
+    QString databaseDrivers() const;
     //! Returns a string containing the available directory drivers
-    virtual QString directoryDrivers() const;
+    QString directoryDrivers() const;
     //! Returns a string containing the available protocol drivers
-    virtual QString protocolDrivers() const;
+    QString protocolDrivers() const;
 
     /**
      * \deprecated since QGIS 3.10 - does nothing - use QgsGui::providerGuiRegistry()
@@ -403,6 +658,11 @@ class CORE_EXPORT QgsProviderRegistry
      */
     QString mMeshDatasetFileFilters;
 
+    /**
+     * File filter string for point cloud files
+     */
+    QString mPointCloudFileFilters;
+
     /**
      * Available database drivers string for vector databases
      *
@@ -426,6 +686,8 @@ class CORE_EXPORT QgsProviderRegistry
      */
     QString mProtocolDrivers;
 
+    QList< UnusableUriHandlerInterface * > mUnusableUriHandlers;
+
     /**
      * Returns TRUE if registry instance exists.
      */
