@@ -1,3 +1,4 @@
+from builtins import object
 # -*- coding: utf-8 -*-
 ###############################################################################
 #
@@ -25,9 +26,11 @@
 
 import logging
 
-from PyQt4.QtCore import QCoreApplication
-from PyQt4.QtGui import QAction, QIcon
+from qgis.PyQt.QtCore import QCoreApplication
+from qgis.PyQt.QtWidgets import QAction
+from qgis.PyQt.QtGui import QIcon
 
+from qgis.core import QgsApplication
 from MetaSearch.dialogs.maindialog import MetaSearchDialog
 from MetaSearch.util import get_help_url, open_url, StaticContext
 
@@ -35,7 +38,6 @@ LOGGER = logging.getLogger('MetaSearch')
 
 
 class MetaSearchPlugin(object):
-
     """base plugin"""
 
     def __init__(self, iface):
@@ -53,13 +55,13 @@ class MetaSearchPlugin(object):
 
         # run
         run_icon = QIcon('%s/%s' % (self.context.ppath,
-                                    'images/MetaSearch.png'))
+                                    'images/MetaSearch.svg'))
         self.action_run = QAction(run_icon, 'MetaSearch',
                                   self.iface.mainWindow())
         self.action_run.setWhatsThis(QCoreApplication.translate('MetaSearch',
                                                                 'MetaSearch plugin'))
         self.action_run.setStatusTip(QCoreApplication.translate('MetaSearch',
-                                                                'Search Metadata Catalogues'))
+                                                                'Search Metadata Catalogs'))
 
         self.action_run.triggered.connect(self.run)
 
@@ -67,7 +69,7 @@ class MetaSearchPlugin(object):
         self.iface.addPluginToWebMenu(self.web_menu, self.action_run)
 
         # help
-        help_icon = QIcon('%s/%s' % (self.context.ppath, 'images/help.png'))
+        help_icon = QgsApplication.getThemeIcon('/mActionHelpContents.svg')
         self.action_help = QAction(help_icon, 'Help', self.iface.mainWindow())
         self.action_help.setWhatsThis(QCoreApplication.translate('MetaSearch',
                                                                  'MetaSearch plugin help'))
