@@ -24,7 +24,6 @@ __copyright__ = '(C) 2014, Alexander Bruy'
 import os
 import random
 
-from qgis.PyQt.QtGui import QIcon
 from qgis.PyQt.QtCore import QVariant
 from qgis.core import (QgsApplication,
                        QgsField,
@@ -35,19 +34,19 @@ from qgis.core import (QgsApplication,
                        QgsPointXY,
                        QgsWkbTypes,
                        QgsSpatialIndex,
-                       QgsFeatureRequest,
                        QgsExpression,
                        QgsDistanceArea,
-                       QgsProject,
+                       QgsPropertyDefinition,
                        QgsProcessing,
                        QgsProcessingException,
-                       QgsProcessingParameterDistance,
+                       QgsProcessingParameters,
+                       QgsProcessingParameterDefinition,
                        QgsProcessingParameterNumber,
+                       QgsProcessingParameterDistance,
                        QgsProcessingParameterFeatureSource,
                        QgsProcessingParameterFeatureSink,
                        QgsProcessingParameterExpression,
-                       QgsProcessingParameterEnum,
-                       QgsProcessingParameterDefinition)
+                       QgsProcessingParameterEnum)
 
 from processing.algs.qgis.QgisAlgorithm import QgisAlgorithm
 from processing.tools import vector
@@ -56,8 +55,8 @@ pluginPath = os.path.split(os.path.split(os.path.dirname(__file__))[0])[0]
 
 
 class RandomPointsPolygons(QgisAlgorithm):
-
     INPUT = 'INPUT'
+    VALUE = 'VALUE'
     EXPRESSION = 'EXPRESSION'
     MIN_DISTANCE = 'MIN_DISTANCE'
     STRATEGY = 'STRATEGY'
@@ -90,12 +89,27 @@ class RandomPointsPolygons(QgisAlgorithm):
                                                      self.strategies,
                                                      False,
                                                      0))
-        self.addParameter(QgsProcessingParameterExpression(self.EXPRESSION,
-                                                           self.tr('Expression'),
-                                                           parentLayerParameterName=self.INPUT))
+        value_param = QgsProcessingParameterNumber(self.VALUE,
+                                                   self.tr('Point count or density'),
+                                                   QgsProcessingParameterNumber.Double,
+                                                   1,
+                                                   minValue=0)
+        value_param.setIsDynamic(True)
+        value_param.setDynamicLayerParameterName(self.INPUT)
+        value_param.setDynamicPropertyDefinition(
+            QgsPropertyDefinition("Value", self.tr("Point count or density"), QgsPropertyDefinition.Double))
+        self.addParameter(value_param)
+
+        # deprecated expression parameter - overrides value parameter if set
+        exp_param = QgsProcessingParameterExpression(self.EXPRESSION,
+                                                     self.tr('Expression'), optional=True,
+                                                     parentLayerParameterName=self.INPUT)
+        exp_param.setFlags(exp_param.flags() | QgsProcessingParameterDefinition.FlagHidden)
+        self.addParameter(exp_param)
+
         self.addParameter(QgsProcessingParameterDistance(self.MIN_DISTANCE,
                                                          self.tr('Minimum distance between points'),
-                                                         0, self.INPUT, False, 0, 1000000000))
+                                                         None, self.INPUT, True, 0, 1000000000))
         self.addParameter(QgsProcessingParameterFeatureSink(self.OUTPUT,
                                                             self.tr('Random points'),
                                                             type=QgsProcessing.TypeVectorPoint))
@@ -112,26 +126,38 @@ class RandomPointsPolygons(QgisAlgorithm):
             raise QgsProcessingException(self.invalidSourceError(parameters, self.INPUT))
 
         strategy = self.parameterAsEnum(parameters, self.STRATEGY, context)
-        minDistance = self.parameterAsDouble(parameters, self.MIN_DISTANCE, context)
-
-        expression = QgsExpression(self.parameterAsString(parameters, self.EXPRESSION, context))
-        if expression.hasParserError():
-            raise QgsProcessingException(expression.parserErrorString())
+        if self.MIN_DISTANCE in parameters and parameters[self.MIN_DISTANCE] is not None:
+            minDistance = self.parameterAsDouble(parameters, self.MIN_DISTANCE, context)
+        else:
+            minDistance = None
 
         expressionContext = self.createExpressionContext(parameters, context, source)
-        expression.prepare(expressionContext)
+        dynamic_value = QgsProcessingParameters.isDynamic(parameters, "VALUE")
+        value_property = None
+        if self.EXPRESSION in parameters and parameters[self.EXPRESSION] is not None:
+            expression = QgsExpression(self.parameterAsString(parameters, self.EXPRESSION, context))
+            value = None
+            if expression.hasParserError():
+                raise QgsProcessingException(expression.parserErrorString())
+            expression.prepare(expressionContext)
+        else:
+            expression = None
+            if dynamic_value:
+                value_property = parameters["VALUE"]
+            value = self.parameterAsDouble(parameters, self.VALUE, context)
 
         fields = QgsFields()
         fields.append(QgsField('id', QVariant.Int, '', 10, 0))
 
         (sink, dest_id) = self.parameterAsSink(parameters, self.OUTPUT, context,
-                                               fields, QgsWkbTypes.Point, source.sourceCrs(), QgsFeatureSink.RegeneratePrimaryKey)
+                                               fields, QgsWkbTypes.Point, source.sourceCrs(),
+                                               QgsFeatureSink.RegeneratePrimaryKey)
         if sink is None:
             raise QgsProcessingException(self.invalidSinkError(parameters, self.OUTPUT))
 
         da = QgsDistanceArea()
         da.setSourceCrs(source.sourceCrs(), context.transformContext())
-        da.setEllipsoid(context.project().ellipsoid())
+        da.setEllipsoid(context.ellipsoid())
 
         total = 100.0 / source.featureCount() if source.featureCount() else 0
         current_progress = 0
@@ -146,12 +172,17 @@ class RandomPointsPolygons(QgisAlgorithm):
             current_progress = total * current
             feedback.setProgress(current_progress)
 
-            expressionContext.setFeature(f)
-            value = expression.evaluate(expressionContext)
-            if expression.hasEvalError():
-                feedback.pushInfo(
-                    self.tr('Evaluation error for feature ID {}: {}').format(f.id(), expression.evalErrorString()))
-                continue
+            this_value = value
+            if value_property is not None or expression is not None:
+                expressionContext.setFeature(f)
+                if value_property:
+                    this_value, _ = value_property.valueAsDouble(expressionContext, value)
+                else:
+                    this_value = expression.evaluate(expressionContext)
+                    if expression.hasEvalError():
+                        feedback.pushInfo(
+                            self.tr('Evaluation error for feature ID {}: {}').format(f.id(), expression.evalErrorString()))
+                        continue
 
             fGeom = f.geometry()
             engine = QgsGeometry.createGeometryEngine(fGeom.constGet())
@@ -159,15 +190,17 @@ class RandomPointsPolygons(QgisAlgorithm):
 
             bbox = fGeom.boundingBox()
             if strategy == 0:
-                pointCount = int(value)
+                pointCount = int(this_value)
             else:
-                pointCount = int(round(value * da.measureArea(fGeom)))
+                pointCount = int(round(this_value * da.measureArea(fGeom)))
 
             if pointCount == 0:
                 feedback.pushInfo("Skip feature {} as number of points for it is 0.".format(f.id()))
                 continue
 
-            index = QgsSpatialIndex()
+            index = None
+            if minDistance:
+                index = QgsSpatialIndex()
             points = dict()
 
             nPoints = 0
@@ -187,14 +220,15 @@ class RandomPointsPolygons(QgisAlgorithm):
                 p = QgsPointXY(rx, ry)
                 geom = QgsGeometry.fromPointXY(p)
                 if engine.contains(geom.constGet()) and \
-                        vector.checkMinDistance(p, index, minDistance, points):
+                        (not minDistance or vector.checkMinDistance(p, index, minDistance, points)):
                     f = QgsFeature(nPoints)
                     f.initAttributes(1)
                     f.setFields(fields)
                     f.setAttribute('id', pointId)
                     f.setGeometry(geom)
                     sink.addFeature(f, QgsFeatureSink.FastInsert)
-                    index.addFeature(f)
+                    if minDistance:
+                        index.addFeature(f)
                     points[nPoints] = p
                     nPoints += 1
                     pointId += 1
