@@ -30,6 +30,7 @@ from qgis.core import (QgsRasterLayer,
                        QgsExpression,
                        QgsProject,
                        QgsRectangle,
+                       QgsWkbTypes,
                        QgsVectorFileWriter,
                        QgsProcessing,
                        QgsProcessingUtils,
@@ -42,6 +43,7 @@ from qgis.core import (QgsRasterLayer,
                        QgsProcessingParameterCrs,
                        QgsProcessingParameterRange,
                        QgsProcessingParameterPoint,
+                       QgsProcessingParameterGeometry,
                        QgsProcessingParameterEnum,
                        QgsProcessingParameterExtent,
                        QgsProcessingParameterExpression,
@@ -56,7 +58,8 @@ from qgis.core import (QgsRasterLayer,
                        QgsProcessingParameterMapLayer,
                        QgsProcessingParameterMultipleLayers,
                        QgsProcessingParameterFeatureSource,
-                       QgsProcessingParameterNumber)
+                       QgsProcessingParameterNumber,
+                       QgsProcessingParameterColor)
 
 from qgis.PyQt.QtCore import QCoreApplication
 
@@ -73,11 +76,12 @@ PARAMETER_TABLE_FIELD = 'field'
 PARAMETER_EXTENT = 'extent'
 PARAMETER_FILE = 'file'
 PARAMETER_POINT = 'point'
+PARAMETER_GEOMETRY = 'geometry'
 PARAMETER_CRS = 'crs'
 PARAMETER_MULTIPLE = 'multilayer'
 PARAMETER_BAND = 'band'
 PARAMETER_LAYOUTITEM = 'layoutitem'
-PARAMETER_MAP_LAYER = 'Map Layer'
+PARAMETER_MAP_LAYER = 'layer'
 PARAMETER_RANGE = 'range'
 PARAMETER_ENUM = 'enum'
 PARAMETER_MATRIX = 'matrix'
@@ -104,8 +108,10 @@ def getParameterFromString(s, context=''):
                 if len(params) > 3:
                     params[3] = True if params[3].lower() == 'true' else False
             elif clazz == QgsProcessingParameterBand:
-                if len(params) > 3:
-                    params[3] = True if params[3].lower() == 'true' else False
+                if len(params) > 4:
+                    params[4] = True if params[4].lower() == 'true' else False
+                if len(params) > 5:
+                    params[5] = True if params[5].lower() == 'true' else False
             elif clazz == QgsProcessingParameterVectorLayer:
                 if len(params) > 2:
                     params[2] = [int(p) for p in params[2].split(';')]
@@ -114,6 +120,10 @@ def getParameterFromString(s, context=''):
             elif clazz == QgsProcessingParameterMapLayer:
                 if len(params) > 3:
                     params[3] = True if params[3].lower() == 'true' else False
+                    try:
+                        params[4] = [int(p) for p in params[4].split(';')]
+                    except:
+                        params[4] = [getattr(QgsProcessing, p.split(".")[1]) for p in params[4].split(';')]
             elif clazz == QgsProcessingParameterBoolean:
                 if len(params) > 2:
                     params[2] = True if params[2].lower() == 'true' else False
@@ -122,6 +132,14 @@ def getParameterFromString(s, context=''):
             elif clazz == QgsProcessingParameterPoint:
                 if len(params) > 3:
                     params[3] = True if params[3].lower() == 'true' else False
+            elif clazz == QgsProcessingParameterGeometry:
+                if len(params) > 3:
+                    params[3] = True if params[3].lower() == 'true' else False
+                if len(params) > 4:
+                    try:
+                        params[4] = [int(p) for p in params[4].split(';')]
+                    except:
+                        params[4] = [getattr(QgsWkbTypes, p.split(".")[1]) for p in params[4].split(';')]
             elif clazz == QgsProcessingParameterCrs:
                 if len(params) > 3:
                     params[3] = True if params[3].lower() == 'true' else False
@@ -146,7 +164,7 @@ def getParameterFromString(s, context=''):
                     params[3] = True if params[3].lower() == 'true' else False
                 if len(params) > 4:
                     # For multiple values; default value is a list of int
-                    if params[3] == True:
+                    if params[3] is True:
                         params[4] = [int(v) for v in params[4].split(',')] if params[4] is not None else None
                     else:
                         params[4] = int(params[4]) if params[4] is not None else None
@@ -187,6 +205,8 @@ def getParameterFromString(s, context=''):
                     params[5] = True if params[5].lower() == 'true' else False
                 if len(params) > 6:
                     params[6] = True if params[6].lower() == 'true' else False
+                if len(params) > 7:
+                    params[7] = True if params[7].lower() == 'true' else False
             elif clazz == QgsProcessingParameterFile:
                 if len(params) > 2:
                     try:
@@ -214,6 +234,11 @@ def getParameterFromString(s, context=''):
                     params[3] = True if params[3].lower() == 'true' else False
                 if len(params) > 4:
                     params[4] = True if params[4].lower() == 'true' else False
+            elif clazz == QgsProcessingParameterColor:
+                if len(params) > 3:
+                    params[3] = True if params[3].lower() == 'true' else False
+                if len(params) > 4:
+                    params[4] = True if params[4].lower() == 'true' else False
             elif clazz == QgsProcessingParameterFileDestination:
                 if len(params) > 4:
                     params[4] = True if params[4].lower() == 'true' else False
@@ -222,6 +247,8 @@ def getParameterFromString(s, context=''):
             elif clazz == QgsProcessingParameterFolderDestination:
                 if len(params) > 3:
                     params[3] = True if params[3].lower() == 'true' else False
+                if len(params) > 4:
+                    params[4] = True if params[4].lower() == 'true' else False
             elif clazz == QgsProcessingParameterRasterDestination:
                 if len(params) > 3:
                     params[3] = True if params[3].lower() == 'true' else False
